<?php

namespace Drupal\trash;

use Drupal\Core\Entity\RevisionableInterface;
use Drupal\workspaces\WorkspaceAssociationInterface;
use Drupal\workspaces\WorkspaceInterface;

/**
 * Provides an override for core's workspace association service.
 */
class TrashWorkspaceAssociation implements WorkspaceAssociationInterface {

  /**
   * The inner workspace association service.
   *
   * @var \Drupal\workspaces\WorkspaceAssociationInterface
   */
  protected $innerWorkspaceAssociation;

  /**
   * The trash manager.
   *
   * @var \Drupal\trash\TrashManagerInterface
   */
  protected $trashManager;

  /**
   * Constructs a TrashWorkspaceAssociation object.
   *
   * @param \Drupal\workspaces\WorkspaceAssociationInterface $workspace_association
   *   The core workspace association service.
   */
  public function __construct(WorkspaceAssociationInterface $workspace_association, TrashManagerInterface $trash_manager) {
    $this->innerWorkspaceAssociation = $workspace_association;
    $this->trashManager = $trash_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function trackEntity(RevisionableInterface $entity, WorkspaceInterface $workspace) {
    $this->innerWorkspaceAssociation->trackEntity($entity, $workspace);
  }

  /**
   * {@inheritdoc}
   */
  public function workspaceInsert(WorkspaceInterface $workspace) {
    $this->innerWorkspaceAssociation->workspaceInsert($workspace);
  }

  /**
   * {@inheritdoc}
   */
  public function getTrackedEntities($workspace_id, $entity_type_id = NULL, $entity_ids = NULL, $offset = NULL, $limit = NULL) {
    return $this->innerWorkspaceAssociation->getTrackedEntities($workspace_id, $entity_type_id, $entity_ids, $offset, $limit);
  }

  /**
   * {@inheritdoc}
   */
  public function getAssociatedRevisions($workspace_id, $entity_type_id, $entity_ids = NULL) {
    return $this->innerWorkspaceAssociation->getAssociatedRevisions($workspace_id, $entity_type_id, $entity_ids);
  }

  /**
   * {@inheritdoc}
   */
  public function getEntityTrackingWorkspaceIds(RevisionableInterface $entity) {
    return $this->innerWorkspaceAssociation->getEntityTrackingWorkspaceIds($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function postPublish(WorkspaceInterface $workspace) {
    $this->innerWorkspaceAssociation->postPublish($workspace);
  }

  /**
   * {@inheritdoc}
   */
  public function deleteAssociations($workspace_id, $entity_type_id = NULL, $entity_ids = NULL) {
    $this->innerWorkspaceAssociation->deleteAssociations($workspace_id, $entity_type_id, $entity_ids);
  }

  /**
   * {@inheritdoc}
   */
  public function initializeWorkspace(WorkspaceInterface $workspace) {
    $this->innerWorkspaceAssociation->initializeWorkspace($workspace);
  }

  /**
   * {@inheritdoc}
   */
  public function isEntityDeletable(RevisionableInterface $entity, WorkspaceInterface $workspace) {
    if ($this->trashManager->isEntityTypeEnabled($entity->getEntityType())) {
      return TRUE;
    }

    if (method_exists($this->innerWorkspaceAssociation, 'isEntityDeletable')) {
      return $this->innerWorkspaceAssociation->isEntityDeletable($entity, $workspace);
    }

    return FALSE;
  }

}
