<?php

namespace Drupal\Tests\trash\Kernel;

use Drupal\trash\Entity\Query\Sql\Query;
use Drupal\trash_test\Entity\TrashTest;

/**
 * Class EntityQueryTest.
 *
 * @group trash
 */
class EntityQueryTest extends TrashKernelTestBase {

  public function testQueryWithoutDeletedAccess() {
    $entities = [];
    $entity_storage = \Drupal::entityTypeManager()->getStorage('trash_test_entity');

    for ($i = 0; $i < 5; $i++) {
      $entity = TrashTest::create();
      $entity->save();
      $entities[] = $entity;
    }

    // Test whether they appear in an entity query.
    $this->assertCount(5, $entity_storage->getQuery()->accessCheck(FALSE)->execute());

    // Delete the first three of them. They should all be individual loadable
    // but no longer accessible via the entity query.
    for ($i = 0; $i < 3; $i++) {
      $entities[$i]->delete();

      $this->assertNotNull(TrashTest::load($entities[$i]->id()));
    }
    $this->assertCount(2, $entity_storage->getQuery()->accessCheck(FALSE)->execute());
  }

  public function testQueryWithDeletedAccess() {
    $entities = [];
    $entity_type_manager = \Drupal::entityTypeManager();
    $entity_storage = $entity_type_manager->getStorage('trash_test_entity');
    $entity_type = $entity_type_manager->getDefinition('trash_test_entity');

    for ($i = 0; $i < 5; $i++) {
      $entity = TrashTest::create();
      $entity->save();
      $entities[] = $entity;
    }

    // Test whether they appear in an entity query.
    $this->assertCount(5, $entity_storage->getQuery()->accessCheck(FALSE)->execute());

    // Delete the first three of them. They should all be individual loadable
    // but not longer accessible via the entity query.
    for ($i = 0; $i < 3; $i++) {
      $entities[$i]->delete();
    }

    // Try to fetch deleted content.
    $query = $entity_storage->getQuery();
    assert($query instanceof Query);
    $query->accessCheck(FALSE);
    $query->isDeleted();
    $this->assertCount(3, $query->execute());

    // Try the same but with using a direct condition on the deleted flag.
    $query = $entity_storage->getQuery();
    assert($query instanceof Query);
    $query->accessCheck(FALSE);
    $query->condition('deleted', 0, '>');
    $this->assertCount(3, $query->execute());

    // Try to fetch not deleted content.
    $query = $entity_storage->getQuery();
    assert($query instanceof Query);
    $query->accessCheck(FALSE);
    $query->isNotDeleted();
    $this->assertCount(2, $query->execute());

    // Try the same but with using a direct condition on the deleted flag.
    $query = $entity_storage->getQuery();
    assert($query instanceof Query);
    $query->accessCheck(FALSE);
    $query->condition('deleted', 0, '>');
    $this->assertCount(3, $query->execute());
  }

}
